@@ -48,7 +48,7 @@ class SvgTemplates:
         else:
             raise BadTemplateException
 
-    def _build_headers(self, name, config, domain="Enterprise", version="8", desc=None, filters=None, gradient=None):
+    def _build_headers(self, name, config, domain="Enterprise", version="8", desc=None, filters=None, gradient=None, legend=[]):
         """Build the header blocks for the svg.
 
         :param name: The name of the layer being exported
@@ -58,6 +58,7 @@ class SvgTemplates:
         :param desc: Description of the layer being exported
         :param filters: Any filters applied to the layer being exported
         :param gradient: Gradient information included with the layer
+        :param legend: List of legend items
         :return: Instantiated SVG header
         """
         max_x = convertToPx(config.width, config.unit)
@@ -84,14 +85,12 @@ class SvgTemplates:
                 header_count += 1
             if config.showDomain:
                 header_count += 1
-            if config.showLegend and gradient is not False and config.legendDocked:
+            if config.showLegend and config.legendDocked and (gradient is not False or legend):
                 header_count += 1
             if self.lhandle.layout:
                 if self.lhandle.layout.showAggregateScores:
                     showAgg = True
                     header_count += 1
-                if config.showFilters:
-                    header_count -= 1
 
             operation_x = (max_x - border) - (1.5 * border * (header_count - 1)) - border
             if header_count > 0:
@@ -171,15 +170,25 @@ class SvgTemplates:
                     header.append(bA)
                     bA.append(gA)
                     psych += 1
-                if config.showLegend and gradient is not False:
-                    gr = gradient
-                    if gr is None:
-                        gr = Gradient(colors=["#ff6666", "#ffe766", "#8ec843"], minValue=1, maxValue=100)
-                    colors = []
-                    div = round((gr.maxValue - gr.minValue) / (len(gr.colors) * 2 - 1))
-                    for i in range(0, len(gr.colors) * 2 - 1):
-                        colors.append((gr.compute_color(int(gr.minValue + div * i)), gr.minValue + div * i))
-                    colors.append((gr.compute_color(gr.maxValue), gr.maxValue))
+                if config.showLegend:
+                    # get all gradient colors
+                    gradient_colors = []
+                    if gradient is not False:
+                        gr = gradient
+                        if gr is None:
+                            gr = Gradient(colors=["#ff6666", "#ffe766", "#8ec843"], minValue=1, maxValue=100)
+                        div = round((gr.maxValue - gr.minValue) / (len(gr.colors) * 2 - 1))
+                        for i in range(0, len(gr.colors) * 2 - 1):
+                            gradient_colors.append((gr.compute_color(int(gr.minValue + div * i)), gr.minValue + div * i))
+                        gradient_colors.append((gr.compute_color(gr.maxValue), gr.maxValue))
+                    
+                    # get all legend colors
+                    legend_colors = []
+                    if legend:
+                        for l in legend:
+                            legend_colors.append((l.color, l.label))
+
+                    # build gradient/legend
                     if config.legendDocked:
                         b3 = G(tx=operation_x / header_count * psych + 1.5 * border * psych)
                         g3 = SVG_HeaderBlock().build(
@@ -187,7 +196,8 @@ class SvgTemplates:
                             width=header_width,
                             label="legend",
                             variant="graphic",
-                            colors=colors,
+                            gradient_colors=gradient_colors,
+                            legend_colors=legend_colors,
                             config=config,
                         )
                         header.append(b3)
@@ -201,7 +211,8 @@ class SvgTemplates:
                             width=adjusted_width,
                             label="legend",
                             variant="graphic",
-                            colors=colors,
+                            gradient_colors=gradient_colors,
+                            legend_colors=legend_colors,
                             config=config,
                         )
                         lx = convertToPx(config.legendX, config.unit)
@@ -217,9 +228,7 @@ class SvgTemplates:
             d.append(root)
         return d, psych, overlay
 
-    def get_tactic(
-        self, tactic, height, width, config, colors=[], scores=[], subtechs=[], exclude=[], mode=(True, False)
-    ):
+    def get_tactic(self, tactic, height, width, config, colors=[], scores=[], subtechs=[], exclude=[], mode=(True, False)):
         """Build a 'tactic column' svg object.
 
         :param tactic: The corresponding tactic for this column
@@ -233,47 +242,56 @@ class SvgTemplates:
         :param mode: Tuple describing text for techniques (Show Name, Show ID)
         :return: Instantiated tactic column (or none if no techniques were found)
         """
-        offset = 0
+        # create tactic column
         column = G(ty=2)
-        for a in tactic.subtechniques:
-            self._copy_scores(tactic.subtechniques[a], scores, tactic.tactic.name, exclude)
-        for x in tactic.techniques:
-            if any(x.id == y[0] and (y[1] == self.h.convert(tactic.tactic.name) or not y[1]) for y in exclude):
+        tactic_name = tactic.tactic.name
+        excluded_ids = [str(t[0]) + str(t[1]) for t in exclude]
+        subtech_ids = [str(s[0]) + str(s[1]) for s in subtechs]
+    
+        # copy scores to SVG
+        offset = 0
+        for id in tactic.subtechniques:
+            self._copy_scores(tactic.subtechniques[id], scores, tactic_name, exclude)
+
+        for technique in tactic.techniques:
+            if (str(technique.id) + str(self.h.convert(tactic_name))) in excluded_ids:
                 continue
-            self._copy_scores([x], scores, tactic.tactic.name, exclude)
-            if any(x.id == y[0] and (y[1] == self.h.convert(tactic.tactic.name) or not y[1]) for y in subtechs):
-                a, offset = self.get_tech(
+            self._copy_scores([technique], scores, tactic_name, exclude)
+            if (str(technique.id) + str(self.h.convert(tactic_name))) in subtech_ids:
+                technique_svg, offset = self.get_tech(
                     offset,
                     mode,
-                    x,
-                    tactic=self.h.convert(tactic.tactic.name),
-                    subtechniques=tactic.subtechniques.get(x.id, []),
+                    technique,
+                    tactic=self.h.convert(tactic_name),
+                    subtechniques=tactic.subtechniques.get(technique.id, []),
+                    exclude=exclude,
                     colors=colors,
                     config=config,
                     height=height,
                     width=width,
-                    subscores=tactic.subtechniques.get(x.id, []),
+                    subscores=tactic.subtechniques.get(technique.id, []),
                 )
             else:
-                a, offset = self.get_tech(
+                technique_svg, offset = self.get_tech(
                     offset,
                     mode,
-                    x,
-                    tactic=self.h.convert(tactic.tactic.name),
+                    technique,
+                    tactic=self.h.convert(tactic_name),
                     subtechniques=[],
+                    exclude=exclude,
                     colors=colors,
                     config=config,
                     height=height,
                     width=width,
-                    subscores=tactic.subtechniques.get(x.id, []),
+                    subscores=tactic.subtechniques.get(technique.id, []),
                 )
-            column.append(a)
+            column.append(technique_svg)
         if len(column.children) == 0:
             return None
         return column
 
     def get_tech(
-        self, offset, mode, technique, tactic, config, height, width, subtechniques=[], colors=[], subscores=[]
+        self, offset, mode, technique, tactic, config, height, width, subtechniques=[], exclude=[], colors=[], subscores=[]
     ):
         """Retrieve a svg object for a single technique.
 
@@ -285,6 +303,7 @@ class SvgTemplates:
         :param height: The allocated height of a technique block
         :param width: The allocated width of a technique block
         :param subtechniques: A list of all visible subtechniques, some of which may apply to this one
+        :param exclude: List of excluded techniques
         :param colors: A list of all color overrides in the event of no score, which may apply
         :param subscores: List of all subtechniques for the (visible or not) [includes scores]
         :return: Tuple (SVG block, new offset)
@@ -300,6 +319,7 @@ class SvgTemplates:
             height,
             width,
             subtechniques=subtechniques,
+            exclude=exclude,
             mode=mode,
             tactic=tactic,
             colors=colors,
@@ -307,73 +327,102 @@ class SvgTemplates:
         )
         return a, b
 
-    def export(self, showName, showID, lhandle, config, sort=0, scores=[], colors=[], subtechs=[], exclude=[]):
+    def export(self, showName, showID, layer, config, sort=0, scores=[], colors=[], subtechs=[], exclude=[], legend=[]):
         """Export a layer object to an SVG object.
 
         :param showName: Boolean of whether or not to show names
         :param showID:  Boolean of whether or not to show IDs
-        :param lhandle: The layer object being exported
+        :param layer: The layer object being exported
         :param config: A SVG Config object
         :param sort: The sort mode
         :param scores: List of tactic scores
         :param colors: List of tactic default colors
         :param subtechs: List of visible subtechniques
         :param exclude: List of excluded techniques
+        :param legend: List of legend items
         :return:
         """
-        self.codex = self.h.get_matrix(self.mode, filters=lhandle.filters)
-        grad = False
+        # get the matrix list of tactics
+        self.matrix = self.h.get_matrix(self.mode, filters=layer.filters)
+
+        # check for a gradient
+        gradient = False
         if len(scores):
-            grad = lhandle.gradient
-        self.lhandle = lhandle
-        d, presence, overlay = self._build_headers(
-            lhandle.name, config, lhandle.domain, lhandle.versions.attack, lhandle.description, lhandle.filters, grad
+            gradient = layer.gradient
+        self.lhandle = layer
+
+        # build SVG headers
+        drawing, presence, overlay = self._build_headers(
+            layer.name, config, layer.domain, layer.versions.attack, layer.description, layer.filters, gradient, legend
         )
-        self.codex = self.h._adjust_ordering(self.codex, sort, scores)
-        index = 0
+
+        # sort matrix by the given sort mode
+        self.matrix = self.h._adjust_ordering(self.matrix, sort, scores)
+
+        # count number of included techniques for each tactic
         lengths = []
+        for tactic in self.matrix:
+            num_techniques = len(tactic.techniques)
+
+            tactic_technique_ids = [technique.id for technique in tactic.techniques]
+            for technique_id, shortname in exclude:
+                if technique_id in tactic_technique_ids:
+                    if self.h.convert(shortname) == tactic.tactic.name or shortname is False:
+                        num_techniques -= 1
+
+            subtech_ids = [subtechnique[0] for subtechnique in subtechs]
+            for subtechnique in tactic.subtechniques:
+                if subtechnique in subtech_ids:
+                    num_techniques += len(tactic.subtechniques[subtechnique])
+
+            lengths.append(num_techniques)
+
+        # calculate border
         border = convertToPx(config.border, config.unit)
-        glob = G(tx=border)
-        for x in self.codex:
-            su = len(x.techniques)
-            for enum in exclude:
-                if enum[0] in [y.id for y in x.techniques]:
-                    if self.h.convert(enum[1]) == x.tactic.name or enum[1] is False:
-                        su -= 1
-            for y in x.subtechniques:
-                if y in [z[0] for z in subtechs]:
-                    su += len(x.subtechniques[y])
-            lengths.append(su)
-        tech_width = (
-            (convertToPx(config.width, config.unit) - 2.2 * border) / sum([1 for x in lengths if x > 0])
-        ) - border
+
+        # calculate technique width
+        technique_width = (convertToPx(config.width, config.unit) - 2.2 * border) / sum([1 for x in lengths if x > 0])
+        technique_width -= border
+
+        # calculate header offset
         header_offset = convertToPx(config.headerHeight, config.unit)
         if presence == 0:
             header_offset = 0
         header_offset += 2.5 * border
-        tech_height = (
-            convertToPx(config.height, config.unit) - header_offset - convertToPx(config.border, config.unit)
-        ) / (max(lengths) + 1)
-        incre = tech_width + 1.1 * border
-        for x in self.codex:
-            disp = ""
+
+        # calculate technique height
+        technique_height = convertToPx(config.height, config.unit) - header_offset - border
+        technique_height /= (max(lengths) + 1)
+
+        # create SVG
+        svg_glob = G(tx=border)
+
+        # build SVG
+        index = 0
+        incre = technique_width + 1.1 * border
+        for tactic in self.matrix:
+            # get tactic display string
+            displayStr = ""
             if showName and showID:
-                disp = x.tactic.id + ": " + x.tactic.name
+                displayStr = tactic.tactic.id + ": " + tactic.tactic.name
             elif showName:
-                disp = x.tactic.name
+                displayStr = tactic.tactic.name
             elif showID:
-                disp = x.tactic.id
+                displayStr = tactic.tactic.id
 
-            g = G(tx=index, ty=header_offset)
+            # create header text
+            header_glob = G(tx=index, ty=header_offset)
+            text_glob = G(tx=technique_width / 2, ty=technique_height / 2)
+            font_size, _ = _optimalFontSize(displayStr, technique_width, technique_height, maxFontSize=28)
+            text = Text(ctype="TacticName", font_size=font_size, text=displayStr, position="middle")
+            text_glob.append(text)
+            header_glob.append(text_glob)
 
-            fs, _ = _optimalFontSize(disp, tech_width, tech_height, maxFontSize=28)
-            tx = Text(ctype="TacticName", font_size=fs, text=disp, position="middle")
-            gt = G(tx=tech_width / 2, ty=tech_height / 2)
-            gt.append(tx)
-            a = self.get_tactic(
-                x,
-                tech_height,
-                tech_width,
+            # get tactic column
+            tactic_col_svg = self.get_tactic(
+                tactic,
+                technique_height,
+                technique_width,
                 colors=colors,
                 subtechs=subtechs,
                 exclude=exclude,
@@ -381,17 +430,23 @@ class SvgTemplates:
                 scores=scores,
                 config=config,
             )
-            b = G(ty=tech_height)
-            g.append(gt)
-            b.append(a)
-            g.append(b)
-            if a:
-                glob.append(g)
+
+            # build tactic column svg
+            tactic_col_glob = G(ty=technique_height)
+            tactic_col_glob.append(tactic_col_svg)
+            header_glob.append(tactic_col_glob)
+
+            if tactic_col_svg:
+                svg_glob.append(header_glob)
                 index += incre
-        d.append(glob)
+
+        drawing.append(svg_glob)
+
+        # add overlay, if applicable
         if overlay:
-            d.append(overlay)
-        return d
+            drawing.append(overlay)
+
+        return drawing
 
     def _copy_scores(self, listing, scores, tactic_name, exclude):
         """Move scores over from the input object (scores) to the one used to build the svg (listing).
@@ -401,8 +456,9 @@ class SvgTemplates:
         :param exclude: List of excluded techniques
         :return: None - operates on the raw object itself
         """
+        excluded_ids = [str(t[0]) + str(t[1]) for t in exclude]
         for b in listing:
-            if b in exclude:
+            if (str(b.id) + str(tactic_name)) in excluded_ids:
                 b.score = None
                 continue
             found = False
